# Generated by Django 4.1.1 on 2022-10-05 01:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Menu',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('link', models.CharField(blank=True, max_length=255, null=True)),
                ('order_menu', models.SmallIntegerField(default=0)),
                ('icon', models.CharField(blank=True, max_length=50, null=True)),
                ('kind', models.SmallIntegerField(choices=[(1, 'Frontend'), (2, 'Backend')], default=1)),
                ('is_visibled', models.BooleanField(default=True)),
                ('is_external', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='MenuGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='MenuCustom',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('menu', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='menu.menu')),
                ('menu_group', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='menu.menugroup')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
            ],
        ),
        migrations.AddField(
            model_name='menu',
            name='menu_group',
            field=models.ManyToManyField(blank=True, null=True, to='menu.menugroup'),
        ),
        migrations.AddField(
            model_name='menu',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='menu.menu'),
        ),
    ]



# from django.shortcuts import render

from django.views.generic import TemplateView

class IndexView(TemplateView):
    template_name = "menu/index.html"

class TestMenuView(TemplateView):
    template_name = 'menu/test-menu.html'


from django.contrib import admin
from .models import Menu, MenuGroup, MenuCustom

class MenuAdmin(admin.ModelAdmin):    
    list_filter = ('name', 'parent',) 
    list_display = ['name', 'parent', 'link', 'order_menu', 'kind', 'updated_at']
    search_fields = ('name', 'parent')
    ordering = ('-updated_at',)

admin.site.register(Menu, MenuAdmin)

class MenuGroupAdmin(admin.ModelAdmin):
    list_filter = ('name',) 
    list_display = ['name', 'updated_at']
    search_fields = ('name',)
    ordering = ('-updated_at',)

admin.site.register(MenuGroup, MenuGroupAdmin)

class MenuCustomAdmin(admin.ModelAdmin):
    list_filter = ('menu',) 
    list_display = ['menu', 'menu_group', 'updated_at']
    search_fields = ('menu',)
    ordering = ('-updated_at',)

admin.site.register(MenuCustom, MenuCustomAdmin)


'''
    1. ----------------------------------------------------------------------------
    Parameter menu_group [Optional]
    Jika kosong, cek Site_ID (proses berdasarkan site ID menu default)
    Jika menu_group kosong, SITE_id harus ada, jika tidak maka ditidak datap di proses

    2. ----------------------------------------------------------------------------
    Di tabel tambahkan User_ID dan Site_ID
    Jika data SITE_ID saja yg ada maka ini untuk front end
    Jika User_ID saja yg ada maka ini sebagai backend

    3. ----------------------------------------------------------------------------
    Jika dua2nya ada, maka USER_ID saja yg di cek SITE_ID tidak di gunakan

    Update 4 Oktober 2022
'''

from django import template
from ..models import Menu
from ..menus import Menus

register = template.Library()

# SET GLOBAL VARIABLE FORM MENU
 # cache nanti di template
 # variable global ini bukan untuk cache
global_menu = {} 
# format:
    # USER_ID, MENU_DATA, User_group, site_ID
    # Ada project_ID (untuk membedakan, project company_profile, acounting, etc)


# 2. Create Menu 
class MenuCreate(template.Node):
    '''
        menu_kind :
            FRONTEND = 1
            BACKEND = 2 

        menu_group :
            jika = 0 maka front end, menu group kosong (ignore untuk mode frontend)
    '''
    def __init__(self, menu_kind, menu_group = 0, var_name='menu'):       
        self.menu_group = menu_group
        self.menu_kind = menu_kind           
        self.var_name = var_name

    # 1. Cache server
    #    Cek jika sudah ada data di variable maka tidak perlu ambil lagi di database
    # def menu_cache():
    #     lanjut = False
    #     if global_menu:
    #         if global_menu[self.menu_group]:

    #     if not global_menu:
    #         if self.menu_kind == 'FRONTEND':                  
    #             my_menu = Menus(self.menu_group, 1)
    #         else:                  
    #             my_menu = Menus(self.menu_group, 2)
    #         global_menu[self.menu_group] = my_menu
    #         print('load from source')
    #     else:
    #         if not global_menu[self.menu_group]:
    #             if self.menu_kind == 'FRONTEND':                  
    #                 my_menu = Menus(self.menu_group, 1)
    #             else:                  
    #                 my_menu = Menus(self.menu_group, 2)
    #             global_menu[self.menu_group] = my_menu
    #             print('load from source')
    #         else:
    #             my_menu = global_menu[self.menu_group]
    #             print('load from cache')   

    def render(self, context):        
        # print('group = ', self.menu_group)
        # print('kinds = ', self.menu_kind)
        # print('var_name = ', self.var_name)
        
        # Parameter di template tidak perlu menggunakan tanda petik untuk menandakan string

        # if not global_menu:
        #     if self.menu_kind == 'FRONTEND':                  
        #         my_menu = Menus(self.menu_group, 1)
        #     else:                  
        #         my_menu = Menus(self.menu_group, 2)
        #     global_menu[self.menu_group] = my_menu
        #     print('load from source')
        # else:
        #     if not global_menu[self.menu_group]:
        #         if self.menu_kind == 'FRONTEND':                  
        #             my_menu = Menus(self.menu_group, 1)
        #         else:                  
        #             my_menu = Menus(self.menu_group, 2)
        #         global_menu[self.menu_group] = my_menu
        #         print('load from source')
        #     else:
        #         my_menu = global_menu[self.menu_group]
        #         print('load from cache') 

        if self.menu_kind == 'FRONTEND':    
            # print('oke')
            my_menu = Menus(self.menu_group, 1)
        else:
            my_menu = Menus(self.menu_group, 2)

        global_menu['0'] = my_menu
        context[self.var_name] = my_menu.get_menus()
        return ''

@register.tag(name='menu_create')            
def get_menu_list(parser, token):
    error = False
    try :
        tag_name, menu_kind, menu_group, _as, var_name = token.split_contents()
        if _as != 'as':
            error = True
    except:
        error = True

    if error:
        raise template.TemplateSyntaxError('menu_create must be of the form, "menu_create <menu_group> <menu_kind> as <var_name>"')
    else:
        return MenuCreate(menu_kind, menu_group, var_name)    

# ------------------------
# get active mnu

# @register.simple_tag(takes_context=True)
# def menu_active_simple_tag(context, active_menu, menu_group):    
#     my_menu = global_menu['1']
#     print('simple tag = ', my_menu.find_activeMenuList(active_menu))
#     return my_menu.get_menus()

# # OKE pakai simple tag
@register.simple_tag    #(takes_context=True) context, 
def menu_active(active_menu):    
    my_menu = global_menu['0']  # harus tipe data string    
    my_menu.get_active_menu(active_menu)
    return my_menu.get_list_active()

# @register.simple_tag # (takes_context=True) context
# def menu_breadcrumb(active_menu):    
#     # print('active_menu =' ,active_menu)
#     my_menu = global_menu['0']  # harus tipe data string  
#     # print(my_menu)  
#     return my_menu.create_breadcrumb(active_menu)


# OKE pakai simple tag untuk create bread crumb
# @register.simple_tag
# def menu_bread_crumb():
#     my_menu = global_menu['0']
#     print('bread  =', my_menu.create_breadCrumb('profile')    )
#     return my_menu.create_breadCrumb('profile')    


# Generated by Django 4.1.1 on 2022-10-07 02:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0003_alter_menu_menu_group'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menucustom',
            name='menu_group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='menu.menugroup'),
        ),
    ]



'''
    Recreate model base on OPD project and Outbox Project
    29th September 2022
    Grid Software, Inc.
'''
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.db import models

User = get_user_model()

class OptMenuKinds(models.IntegerChoices):
    '''
        Jenis menu : 
            0 : Front end without user login
            1 : Front end with user login
            2 : Back end with user login (nothing for none user)

            Update 17 Sept 2021
            0 : Front End
            1 : Back End
    '''
    FRONTEND = 1
    BACKEND = 2 
    # Tidak jadi karena crash dengan unique together di model
    # BACKEND_DEFAULT = 3 # Penanda user baru login, belum ada company yg aktif, sehingga menu belum bisa di generate
                        # untuk itu generate menu default langsung   


class MenuGroup(models.Model):
    '''
        Group : Model Menu
        Simpan data group menu :
        1. Owner
        2. Manager
        3. Operator
        4. Kasir
        5. dll...

        Tidak ada interface, maintenance di halaman admin
    '''
    # Tidak boleh ada data kembar di name
    name = models.CharField(max_length=100, unique=True)

    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)    

    # Optional Fields:
    # created_at = models.DateTimeField(auto_now_add=True)
    # updated_at = models.DateTimeField(auto_now=True)

    # objects = MenuGroupManager()
    
    def __str__(self):
        return self.name

# def company_name_validate(value):
#     if len(value) < 3:
#         raise ValidationError("Company name mush be more than 3 character")
#     else:
#         return value

class Menu(models.Model):
    '''    
        Relase :
            User --1:N-- Menu
        Deskripsi :
            # Menu di buat per user,
            # Satu company bisa mempunyai banyak user,
            # Satu User bisa punya banyak menu
            # Tetapkan satu user sebagai user master (berguna sebagai menu master), di copy ke menu user lain
    
        # ada 1 company dengan site ID = 1 sebagai master menu
        # seluruh menu di create dari site ini (berarti is_master_menu tidak di perlukan lagi)
        # Untuk menu di app blog, user tidak dalam kondisi login
        # Untuk menu di app inventory, user harus login dulu        
        # Jika user telah di create, langsung generate menu untuk user tersebut
        # dapat di visible sesuai kebutuhan
    '''       
    name = models.CharField(max_length=100)

    # menu group, relasi many to many 
    # Untuk frontend menu group kosong
    menu_group = models.ManyToManyField(MenuGroup, blank=True) # , null=True, blank=True)  not effect to m2m relation

    # Optional Fields:
    # user = models.ForeignKey(User, null=True, blank=True, on_delete=models.PROTECT)        # Menu di generate per user   
    # user harus diimport dulu

    parent = models.ForeignKey('self', null=True, blank=True, on_delete=models.CASCADE)    

    # link: adalah name menu yg di slugify
    # href di ubah menjadi link
    # jika ada ://www maka dianggap link luar
    # /home/dashboard is not valid URLField (change back to CharField)
    # link = models.URLField(max_length=255, null=True, blank=True)  # , verbose_name='Link'
    link = models.CharField(max_length=255, null=True, blank=True) 

    # urut menu
    order_menu = models.SmallIntegerField(default=0)

    # awesome icon
    # data berupa blok <i class="icon"></i>
    icon = models.CharField(max_length=50, null=True, blank=True)    

    #is_admin_menu = models.BooleanField(default=False)	    
    # is_admin_menu: jika true, menu untuk backend, jika false menu untuk frontend
    # jenis menu
    kind = models.SmallIntegerField(choices=OptMenuKinds.choices, default=OptMenuKinds.FRONTEND)

    # is_visibled: untuk menyembunyikan menu
    is_visibled = models.BooleanField(default=True)	    

    # is_master_menu: adalah menu yg digunakan oleh banyak user
    # tidak bisa dihapus, hanya super user yg bisa menghapus
    # is_master_menu adalah menu yg di copy oleh user lain saat pertama kali di buat

    # Update 17 Sept 2021 
    # is_master_menu tidak digunakan lagi karena seluruh menu di create sesuai dengan
    # data menu di SITE 1
    # (menu di SITE 1 sebagai menu master yg akan di copy ke site lain saat User pertama kali di create)
    # is_master_menu = models.BooleanField(default=False)	 

    # is_statis_menu: penanda halaman statis
    # semua menu yg di create oleh user adalah statis menu
    # diakses khusus melalui link tertentu di halaman statis   
    # is_statis_menu = models.BooleanField(default=False)	
    is_external = models.BooleanField(default=False)	    # Jika external menu True, maka otomatis target _BLANK

    # timestamp
    # created_at = models.DateTimeField(auto_now_add=True)
    # updated_at = models.DateTimeField(auto_now=True)

    # untuk status sinkronisasi ke server
    # penanda di server lokal apakah sudah di sinkronisasi atau belum
    # yg paling penting tanggal harus valid 
    # is_already_sync = models.BooleanField(default=False)	

    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)    

    # def __str__(self):
    #     return self.name

    # class Meta:
    #     constraints = [
    #         models.UniqueConstraint(fields=['menu_group', 'name'], name='unique_menu_group_name')
    #     ]

    # def updated_at_(self):
    #     return get_natural_datetime(self.updated_at)

    # def created_at_(self):
    #     return get_natural_datetime(self.created_at)

    # def parent_(self):
    #     if self.parent:
    #         return self.parent.name
    #     else:
    #         return '-'

    # def updated_at_(self):
    #     return get_natural_datetime(self.updated_at)        
    
    # def updated_at_(self):
    #     return  "naturalday(self.updated_at)"
       # return serializers.serialize("json",' naturalday(self.updated_at)')
        #return serializers.serialize('json', naturalday(self.updated_at))

    def __str__(self):          
        if self.kind == OptMenuKinds.FRONTEND:
            res = '[ Front-End ]'       # halaman depan
        # elif self.kind == OptMenuKinds.BACKEND_DEFAULT:
        #     res = '[ Default ]'
        else:
            res = '[ Back-End ]'        # halaman dashboard

        if self.parent:
            par = self.parent.name      # tampilkan name parent jika ada    
        else:
            par = 'ROOT'

        return "{} {} > {}".format(res, par, self.name)  


class MenuCustom(models.Model):
    '''
        Custom menu adalah menu yg hanya muncul di site dan menu_group tertentu saja
        tidak muncul di tempat lain
    '''
    site = models.ForeignKey(Site, on_delete=models.CASCADE)

    menu_group = models.ForeignKey(MenuGroup, on_delete=models.PROTECT, blank=True, null=True)    
    menu = models.OneToOneField(Menu, on_delete=models.CASCADE) # One to One Relations to menu

    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)    


    def __str__(self):  
        return "{}".format(self.menu)  



# Generated by Django 4.1.1 on 2022-10-05 01:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0002_alter_menu_menu_group'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menu',
            name='menu_group',
            field=models.ManyToManyField(blank=True, to='menu.menugroup'),
        ),
    ]



from django.urls import path
# from django.views.generic import TemplateView
from .views import IndexView, TestMenuView

urlpatterns = [
    # path('', TemplateView.as_view(template_name="about.html")),
    path('', IndexView.as_view(), name='index_view'),
    path('menu/', TestMenuView.as_view(), name='test_menu_view'),
]


from setuptools import setup, find_packages
from os.path import abspath, dirname, join

# Fetches the content from README.md
# This will be used for the "long_description" field.
README_MD = open(join(dirname(abspath(__file__)), "README.md")).read()

setup(
    # The name of your project that we discussed earlier.
    # This name will decide what users will type when they install your package.
    # In my case it will be:
    # pip install pydash-arnu515
    # This field is REQUIRED
    name="django-outbox-menu",

    # The version of your project.
    # Usually, it would be in the form of:
    # major.minor.patch
    # eg: 1.0.0, 1.0.1, 3.0.2, 5.0-beta, etc.
    # You CANNOT upload two versions of your package with the same version number
    # This field is REQUIRED
    version="1.0.1",


    # The packages that constitute your project.
    # For my project, I have only one - "pydash".
    # Either you could write the name of the package, or
    # alternatively use setuptools.findpackages()
    #
    # If you only have one file, instead of a package,
    # you can instead use the py_modules field instead.
    # EITHER py_modules OR packages should be present.

    # use array for exclude for better way to exclude package
    packages=find_packages(exclude=["tests","django_menu"]),

    # dependencies
    # install_requires=[
    #     'cryptography',
    #     'python-decouple',
    # ],

    # The description that will be shown on PyPI.
    # Keep it short and concise
    # This field is OPTIONAL
    description="DJANGO OUTBOX MENU",

    # The content that will be shown on your project page.
    # In this case, we're displaying whatever is there in our README.md file
    # This field is OPTIONAL
    long_description=README_MD,

    # Now, we'll tell PyPI what language our README file is in.
    # In my case it is in Markdown, so I'll write "text/markdown"
    # Some people use reStructuredText instead, so you should write "text/x-rst"
    # If your README is just a text file, you have to write "text/plain"
    # This field is OPTIONAL
    long_description_content_type="text/markdown",

    # The url field should contain a link to a git repository, the project's website
    # or the project's documentation. I'll leave a link to this project's Github repository.
    # This field is OPTIONAL
    url="https://github.com/PROJECT-OUTBOX/django_lib_outbox_menu.git",

    # The author name and email fields are self explanatory.
    # These fields are OPTIONAL
    author_name="ione03",
    author_email="suratiwan03@gmail.com",

    # Classifiers help categorize your project.
    # For a complete list of classifiers, visit:
    # https://pypi.org/classifiers
    # This is OPTIONAL
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3 :: Only"
    ],

    # Keywords are tags that identify your project and help searching for it
    # This field is OPTIONAL
    keywords="menu, model base, environment, django",

    # For additional fields, check:
    # https://github.com/pypa/sampleproject/blob/master/setup.py

    
)
